MODULE TYPES

! Definitions of various derived data types

USE PRECISION_PARAMETERS
USE GLOBAL_CONSTANTS, ONLY : NULL_BOUNDARY,NEUMANN,MAX_SPECIES, &
    IAXIS,JAXIS,KAXIS,MAX_DIM,NOD1,NOD2,IBM_MAX_WSTRIANG_SEG,LOW_IND,HIGH_IND

IMPLICIT NONE

TYPE LAGRANGIAN_PARTICLE_CLASS_TYPE
   CHARACTER(LABEL_LENGTH) :: ID,SPEC_ID,DEVC_ID='null',CTRL_ID='null',QUANTITIES(10),SMOKEVIEW_BAR_LABEL(10),&
                    SURF_ID='null',PROP_ID='null',&
                    RADIATIVE_PROPERTY_TABLE_ID='null',CNF_RAMP_ID='null',DISTRIBUTION='ROSIN-RAMMLER-LOGNORMAL',&
                    BREAKUP_DISTRIBUTION='ROSIN-RAMMLER-LOGNORMAL',BREAKUP_CNF_RAMP_ID='null'
   CHARACTER(60) :: SMOKEVIEW_LABEL(10),QUANTITIES_SPEC_ID(10)
   REAL(EB) :: HEAT_OF_COMBUSTION,ADJUST_EVAPORATION, &
               LIFETIME,DIAMETER,MINIMUM_DIAMETER,MAXIMUM_DIAMETER,GAMMA,KILL_RADIUS,KILL_MASS, &
               TMP_INITIAL,SIGMA,VERTICAL_VELOCITY,HORIZONTAL_VELOCITY,DRAG_COEFFICIENT(3),SURFACE_DIAMETER,&
               SURFACE_TENSION,BREAKUP_RATIO,BREAKUP_GAMMA,BREAKUP_SIGMA,DENSE_VOLUME_FRACTION, PERMEABILITY(3),&
               REAL_REFRACTIVE_INDEX,COMPLEX_REFRACTIVE_INDEX,TOL_INT,DENSITY=-1._EB,FTPR,FREE_AREA_FRACTION,&
               POROUS_VOLUME_FRACTION,MEAN_DROPLET_VOLUME=0._EB,RUNNING_AVERAGE_FACTOR,SHAPE_FACTOR,&
               EMBER_DENSITY_THRESHOLD,EMBER_VELOCITY_THRESHOLD,&
               PRIMARY_BREAKUP_TIME,PRIMARY_BREAKUP_DRAG_REDUCTION_FACTOR
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: R_CNF,CNF,CVF,BREAKUP_R_CNF,BREAKUP_CNF,BREAKUP_CVF,W_CNF,R50,LAMBDA,SOLID_ANGLE
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: WQABS,WQSCA
   INTEGER :: SAMPLING,N_QUANTITIES,QUANTITIES_INDEX(10),QUANTITIES_Y_INDEX(10)=-1,QUANTITIES_Z_INDEX(10)=-1,ARRAY_INDEX=0,&
              RGB(3),RADIATIVE_PROPERTY_INDEX=0,SURF_INDEX=0,DRAG_LAW=1,DEVC_INDEX=0,CTRL_INDEX=0,PROP_INDEX=-1,&
              ORIENTATION_INDEX=0,N_ORIENTATION,Z_INDEX=-1,N_STRATA=6,NEAREST_RAD_ANGLE_INDEX=0, &
              Y_INDEX=-1,CNF_RAMP_INDEX=-1,BREAKUP_CNF_RAMP_INDEX=-1,N_STORAGE_REALS,N_STORAGE_INTEGERS,N_STORAGE_LOGICALS
   INTEGER,  ALLOCATABLE, DIMENSION(:) :: IL_CNF,IU_CNF
   LOGICAL :: STATIC=.FALSE.,MASSLESS_TRACER=.FALSE.,MASSLESS_TARGET=.FALSE.,LIQUID_DROPLET=.FALSE.,SOLID_PARTICLE=.FALSE., &
              MONODISPERSE=.FALSE.,TURBULENT_DISPERSION=.FALSE.,BREAKUP=.FALSE.,CHECK_DISTRIBUTION=.FALSE.,FUEL=.FALSE., &
              PERIODIC_X=.FALSE.,PERIODIC_Y=.FALSE.,PERIODIC_Z=.FALSE.,&
              SECOND_ORDER_PARTICLE_TRANSPORT=.FALSE.,DUCT_PARTICLE=.FALSE.,EMBER_PARTICLE=.FALSE.
END TYPE LAGRANGIAN_PARTICLE_CLASS_TYPE

TYPE (LAGRANGIAN_PARTICLE_CLASS_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: LAGRANGIAN_PARTICLE_CLASS

! The following derived types are used within the ONE_D_M_AND_E_XFER_TYPE

TYPE MATL_COMP_TYPE
   REAL(EB), POINTER, DIMENSION(:) :: RHO
END TYPE MATL_COMP_TYPE

TYPE BAND_TYPE
   REAL(EB), POINTER, DIMENSION(:) :: ILW
END TYPE BAND_TYPE

! Note: If you change the number of scalar variables in ONE_D_M_AND_E_XFER_TYPE, adjust the numbers below

INTEGER, PARAMETER :: N_ONE_D_SCALAR_REALS=30,N_ONE_D_SCALAR_INTEGERS=10,N_ONE_D_SCALAR_LOGICALS=2

TYPE ONE_D_M_AND_E_XFER_TYPE
   REAL(EB), POINTER, DIMENSION(:) :: TMP,LAYER_THICKNESS,X,MASSFLUX_SPEC,MASSFLUX_MATL,MASSFLUX,IL,ZZ_G,ZZ_F,RHO_D_F,RHO_D_DZDN_F,&
                                      A_LP_MPUA,AWM_AEROSOL,LP_CPUA,LP_MPUA
   TYPE(MATL_COMP_TYPE), ALLOCATABLE, DIMENSION(:) :: MATL_COMP
   TYPE(BAND_TYPE), ALLOCATABLE, DIMENSION(:) :: BAND
   INTEGER, POINTER, DIMENSION(:) :: N_LAYER_CELLS
   INTEGER, POINTER :: ARRAY_INDEX,STORAGE_INDEX,II,JJ,KK,IIG,JJG,KKG,IOR,PRESSURE_ZONE
   REAL(EB), POINTER :: AREA,HEAT_TRANS_COEF,Q_CON_F,Q_RAD_IN,Q_RAD_OUT,EMISSIVITY,AREA_ADJUST,T_IGN,TMP_F,TMP_B,&
                        U_NORMAL,U_NORMAL_S,U_NORMAL_0,RSUM_G,TMP_G,RHO_G,U_TANG,RHO_F,RDN,MU_G,K_G,U_TAU,Y_PLUS,Z_STAR,&
                        PHI_LS,WORK1,WORK2,Q_DOT_G_PP,Q_DOT_O2_PP,Q_CONDENSE
   LOGICAL, POINTER :: BURNAWAY,CHANGE_THICKNESS
END TYPE ONE_D_M_AND_E_XFER_TYPE

! Note: If you change the number of scalar variables in LAGRANGIAN_PARTICLE_TYPE, adjust the numbers below

INTEGER, PARAMETER :: N_PARTICLE_SCALAR_REALS=17,N_PARTICLE_SCALAR_INTEGERS=11,N_PARTICLE_SCALAR_LOGICALS=4

TYPE LAGRANGIAN_PARTICLE_TYPE
   TYPE (ONE_D_M_AND_E_XFER_TYPE) :: ONE_D
   LOGICAL, POINTER :: SHOW,SPLAT,EMBER,PATH_PARTICLE
   REAL(EB), POINTER :: X,Y,Z,U,V,W,PWT,ACCEL_X,ACCEL_Y,ACCEL_Z,RE,MASS,T_INSERT,DX,DY,DZ,M_DOT
   INTEGER, POINTER :: TAG,ARRAY_INDEX,STORAGE_INDEX,CLASS_INDEX,ORIENTATION_INDEX,WALL_INDEX,FACE_INDEX,DUCT_INDEX,&
                       INIT_INDEX,DUCT_CELL_INDEX,CFACE_INDEX
END TYPE LAGRANGIAN_PARTICLE_TYPE


TYPE STORAGE_TYPE
   INTEGER :: N_STORAGE_SLOTS=0,NEXT_AVAILABLE_SLOT=1
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: REALS
   INTEGER, ALLOCATABLE, DIMENSION(:,:) :: INTEGERS
   LOGICAL, ALLOCATABLE, DIMENSION(:,:) :: LOGICALS
END TYPE STORAGE_TYPE

! Note: If you change the number of scalar variables in WALL_TYPE, adjust the numbers below

INTEGER, PARAMETER :: N_WALL_SCALAR_REALS=8,N_WALL_SCALAR_INTEGERS=15,N_WALL_SCALAR_LOGICALS=0

TYPE WALL_TYPE
   TYPE (ONE_D_M_AND_E_XFER_TYPE) :: ONE_D
   REAL(EB), POINTER :: DUNDT,EW,Q_LEAK,V_DEP,VEL_ERR_NEW,X,Y,Z
   INTEGER, POINTER :: BACK_INDEX,BACK_MESH,BOUNDARY_TYPE,NODE_INDEX,OBST_INDEX,PRESSURE_BC_INDEX,SURF_INDEX,&
                       SURF_INDEX_ORIG,VENT_INDEX,WALL_INDEX,LAPLACE_BC_INDEX,JD11_INDEX,JD12_INDEX,JD21_INDEX,JD22_INDEX
END TYPE WALL_TYPE

TYPE EXTERNAL_WALL_TYPE
   INTEGER :: NOM,NIC_MIN,NIC_MAX,IIO_MIN,IIO_MAX,JJO_MIN,JJO_MAX,KKO_MIN,KKO_MAX
   REAL(EB) :: AREA_RATIO
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: FVN, FVNS ! species advective flux at E-wall -> flxint(rho*Y_alpha)*U
   ! where flxint(rho*Y_alpha) is the corresponding flux limited interpolation of rho*Y_alpha being used
   ! in the code and U is the normal velocity. FVN -> based on variables at time level n, FVNS based on
   ! predicted variables for step n -> n+1.
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: RHO_D_DZDN, RHO_D_DZDNS ! species diffusive flux at E-wall, as computed in divg.f90.
END TYPE EXTERNAL_WALL_TYPE

TYPE EXPOSED_WALL_TYPE
   REAL(EB) :: Q_RAD_IN,TMP_GAS
END TYPE EXPOSED_WALL_TYPE

TYPE SPECIES_TYPE
   REAL(EB) :: MW=0._EB,YY0=0._EB,RCON,MAXMASS,MASS_EXTINCTION_COEFFICIENT=0._EB,&
               SPECIFIC_HEAT=-1._EB,REFERENCE_ENTHALPY=-1._EB,&
               REFERENCE_TEMPERATURE,MU_USER=-1._EB,K_USER=-1._EB,D_USER=-1._EB,EPSK=-1._EB,SIG=-1._EB,PR_USER=-1._EB,&
               FLD_LETHAL_DOSE=0._EB,FIC_CONCENTRATION=0._EB,&
               SPECIFIC_HEAT_LIQUID=-1,DENSITY_LIQUID,HEAT_OF_VAPORIZATION=-1._EB,&
               H_F,H_V_REFERENCE_TEMPERATURE=-1._EB,H_V_CORRECTOR=0._EB ,TMP_V=-1._EB,TMP_MELT=-1._EB,ATOMS(118)=0._EB,&
               MEAN_DIAMETER=1.E-6_EB,CONDUCTIVITY_SOLID,DENSITY_SOLID,BETA_LIQUID,MU_LIQUID,K_LIQUID,PR_LIQUID, &
               THERMOPHORETIC_DIAMETER=0.03E-6_EB
   LOGICAL ::  ISFUEL=.FALSE.,LISTED=.FALSE.,AGGLOMERATING=.FALSE.,EXPLICIT_H_F=.FALSE.,CONDENSABLE=.FALSE.
   CHARACTER(LABEL_LENGTH) :: ID,RAMP_CP,RAMP_CP_L,RAMP_K,RAMP_MU,RAMP_D,RADCAL_ID,RAMP_G_F,PROP_ID
   CHARACTER(FORMULA_LENGTH) :: FORMULA
   INTEGER :: MODE=2,RAMP_CP_INDEX=-1,RAMP_CP_L_INDEX=-1,RAMP_K_INDEX=-1,RAMP_MU_INDEX=-1,RAMP_D_INDEX=-1,RADCAL_INDEX=-1,&
              RAMP_G_F_INDEX=-1,AWM_INDEX=-1
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: H_V,C_P_L,C_P_L_BAR,H_L

END TYPE SPECIES_TYPE

TYPE (SPECIES_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: SPECIES

TYPE SPECIES_MIXTURE_TYPE
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: MASS_FRACTION,VOLUME_FRACTION
   REAL(EB) :: MW , RCON, ZZ0=0._EB, MASS_EXTINCTION_COEFFICIENT=0._EB,ADJUST_NU=1._EB,ATOMS(118)=0._EB,MEAN_DIAMETER,&
               SPECIFIC_HEAT=-1._EB,REFERENCE_ENTHALPY=-2.E20_EB,THERMOPHORETIC_DIAMETER, &
               REFERENCE_TEMPERATURE,MU_USER=-1._EB,K_USER=-1._EB,D_USER=-1._EB,PR_USER=-1._EB,EPSK=-1._EB,SIG=-1._EB,&
               FLD_LETHAL_DOSE=0._EB,FIC_CONCENTRATION=0._EB,&
               DENSITY_SOLID,CONDUCTIVITY_SOLID,H_F=-1.E30_EB
   CHARACTER(LABEL_LENGTH), ALLOCATABLE, DIMENSION(:) :: SPEC_ID
   CHARACTER(LABEL_LENGTH) :: ID='null',RAMP_CP,RAMP_CP_L,RAMP_K,RAMP_MU,RAMP_D,RAMP_G_F
   CHARACTER(FORMULA_LENGTH) :: FORMULA='null'
   INTEGER :: AWM_INDEX = -1,RAMP_CP_INDEX=-1,SINGLE_SPEC_INDEX=-1,RAMP_K_INDEX=-1,RAMP_MU_INDEX=-1,RAMP_D_INDEX=-1,&
              RAMP_G_F_INDEX=-1,CONDENSATION_SMIX_INDEX=-1,EVAPORATION_SMIX_INDEX=-1,AGGLOMERATION_INDEX=-1
   LOGICAL :: DEPOSITING=.FALSE.,VALID_ATOMS=.TRUE.,EVAPORATING=.FALSE.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: WQABS,WQSCA
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: R50

END TYPE SPECIES_MIXTURE_TYPE

TYPE (SPECIES_MIXTURE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: SPECIES_MIXTURE

TYPE REACTION_TYPE
   CHARACTER(LABEL_LENGTH) :: FUEL,OXIDIZER,PRODUCTS,ID,RAMP_CHI_R
   CHARACTER(LABEL_LENGTH), ALLOCATABLE, DIMENSION(:) :: SPEC_ID_NU,SPEC_ID_NU_READ,SPEC_ID_N_S,SPEC_ID_N_S_READ
   REAL(EB) :: C,H,N,O,EPUMO2,HEAT_OF_COMBUSTION,HOC_COMPLETE,A,A_PRIME,A_PRIME_FAST,A_IN,E,E_IN,K,&
               Y_O2_INFTY,Y_N2_INFTY=0._EB,MW_FUEL,MW_SOOT,Y_O2_MIN,&
               CO_YIELD,SOOT_YIELD,H2_YIELD, SOOT_H_FRACTION,RHO_EXPONENT,RHO_EXPONENT_FAST,CRIT_FLAME_TMP,&
               NU_O2=0._EB,NU_N2=0._EB,NU_H2O=0._EB,NU_H2=0._EB,NU_CO2=0._EB,NU_CO=0._EB,NU_SOOT=0._EB,S=0._EB,&
               COMPLETE_HEAT_OF_COMBUSTION
   REAL(EB) :: N_T=0._EB,CHI_R
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: NU,NU_READ,NU_SPECIES,N_S,N_S_READ,NU_MW_O_MW_F
   INTEGER :: FUEL_SMIX_INDEX=-1,AIR_SMIX_INDEX=-1,N_SMIX,N_SPEC,RAMP_CHI_R_INDEX=0,PRIORITY=1
   LOGICAL :: IDEAL,CHECK_ATOM_BALANCE,FAST_CHEMISTRY=.FALSE.,REVERSE=.FALSE.,THIRD_BODY=.FALSE.
   CHARACTER(MESSAGE_LENGTH) :: FYI='null'
   CHARACTER(255) :: EQUATION
   CHARACTER(100) :: FWD_ID
END TYPE REACTION_TYPE

TYPE (REACTION_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: REACTION

TYPE MATERIAL_TYPE
   REAL(EB) :: K_S,C_S,RHO_S,EMISSIVITY,THERMAL_DIFFUSIVITY,KAPPA_S,TMP_BOIL,REFRACTIVE_INDEX
   INTEGER :: PYROLYSIS_MODEL
   CHARACTER(LABEL_LENGTH) :: ID
   CHARACTER(LABEL_LENGTH) :: RAMP_H_R(MAX_REACTIONS),RAMP_K_S,RAMP_C_S
   INTEGER :: N_REACTIONS,PROP_INDEX=-1,H_R_I(MAX_REACTIONS)=0
   INTEGER, DIMENSION(MAX_REACTIONS) :: N_RESIDUE
   INTEGER, DIMENSION(MAX_MATERIALS,MAX_REACTIONS) :: RESIDUE_MATL_INDEX
   INTEGER, DIMENSION(3) :: RGB
   REAL(EB), DIMENSION(MAX_REACTIONS) :: TMP_REF,TMP_THR,RATE_REF,THR_SIGN
   REAL(EB), DIMENSION(MAX_MATERIALS,MAX_REACTIONS) :: NU_RESIDUE=0._EB
   REAL(EB), DIMENSION(MAX_REACTIONS) :: A,E,H_R,N_S,N_T,N_O2,GAS_DIFFUSION_DEPTH,NU_O2_CHAR,ALPHA_CHAR,BETA_CHAR
   REAL(EB), DIMENSION(MAX_REACTIONS) :: HEATING_RATE,PYROLYSIS_RANGE,TOL_INT
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: NU_GAS,ADJUST_BURN_RATE
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: DIFFUSIVITY_GAS
   REAL(EB), DIMENSION(MAX_SPECIES,MAX_REACTIONS) :: NU_SPEC,HEAT_OF_COMBUSTION
   REAL(EB), DIMENSION(MAX_SPECIES) :: DIFFUSIVITY_SPEC
   LOGICAL, DIMENSION(MAX_REACTIONS) :: PCR = .FALSE.
   LOGICAL :: ALLOW_SHRINKING, ALLOW_SWELLING
   CHARACTER(LABEL_LENGTH), DIMENSION(MAX_MATERIALS,MAX_REACTIONS) :: RESIDUE_MATL_NAME
   CHARACTER(LABEL_LENGTH), DIMENSION(MAX_SPECIES,MAX_REACTIONS) :: SPEC_ID
   CHARACTER(MESSAGE_LENGTH) :: FYI='null'
END TYPE MATERIAL_TYPE

TYPE (MATERIAL_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: MATERIAL

TYPE SURFACE_TYPE
   REAL(EB) :: TMP_FRONT=-1._EB,TMP_BACK=-1._EB,VEL,VEL_GRAD,PLE,Z0,CONVECTIVE_HEAT_FLUX,NET_HEAT_FLUX, &
               VOLUME_FLOW,HRRPUA,MLRPUA,T_IGN,SURFACE_DENSITY,CELL_SIZE_FACTOR, &
               E_COEFFICIENT,TEXTURE_WIDTH,TEXTURE_HEIGHT,THICKNESS,EXTERNAL_FLUX, &
               DXF,DXB,MASS_FLUX_TOTAL,PARTICLE_MASS_FLUX,EMISSIVITY,MAX_PRESSURE, &
               TMP_IGN,TMP_EXT,H_V,LAYER_DIVIDE,ROUGHNESS,LENGTH=-1._EB,WIDTH=-1._EB, &
               DT_INSERT,H_FIXED=-1._EB,H_FIXED_B=-1._EB,HM_FIXED=-1._EB,EMISSIVITY_BACK,CONV_LENGTH,XYZ(3),FIRE_SPREAD_RATE, &
               MINIMUM_LAYER_THICKNESS,INNER_RADIUS=0._EB,MASS_FLUX_VAR=-1._EB,VEL_BULK, &
               PARTICLE_SURFACE_DENSITY=-1._EB,DRAG_COEFFICIENT=2.8_EB,SHAPE_FACTOR=0.25_EB,&
               MINIMUM_BURNOUT_TIME=1.E6_EB
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: DX,RDX,RDXN,X_S,DX_WGT,MF_FRAC,PARTICLE_INSERT_CLOCK
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: RHO_0
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: MASS_FRACTION,MASS_FLUX,TAU,ADJUST_BURN_RATE
   INTEGER,  ALLOCATABLE, DIMENSION(:) :: RAMP_INDEX
   INTEGER, DIMENSION(3) :: RGB
   REAL(EB) :: TRANSPARENCY
   REAL(EB), DIMENSION(2) :: VEL_T
   INTEGER, DIMENSION(2) :: LEAK_PATH,DUCT_PATH
   INTEGER :: THERMAL_BC_INDEX,NPPC,SPECIES_BC_INDEX,VELOCITY_BC_INDEX,SURF_TYPE,N_CELLS_INI,N_CELLS_MAX=0, &
              PART_INDEX,PROP_INDEX=-1,RAMP_T_I_INDEX=-1, RAMP_T_B_INDEX=0
   INTEGER :: PYROLYSIS_MODEL,NRA,NSB
   INTEGER :: N_LAYERS,N_MATL
   INTEGER :: N_ONE_D_STORAGE_REALS,N_ONE_D_STORAGE_INTEGERS,N_ONE_D_STORAGE_LOGICALS
   INTEGER :: N_WALL_STORAGE_REALS,N_WALL_STORAGE_INTEGERS,N_WALL_STORAGE_LOGICALS
   INTEGER :: N_CFACE_STORAGE_REALS,N_CFACE_STORAGE_INTEGERS,N_CFACE_STORAGE_LOGICALS
   INTEGER, DIMENSION(20) :: ONE_D_REALS_ARRAY_SIZE=0,ONE_D_INTEGERS_ARRAY_SIZE=0,ONE_D_LOGICALS_ARRAY_SIZE=0
   INTEGER, ALLOCATABLE, DIMENSION(:) :: N_LAYER_CELLS,LAYER_INDEX,MATL_INDEX
   INTEGER, DIMENSION(MAX_LAYERS,MAX_MATERIALS) :: LAYER_MATL_INDEX
   INTEGER, DIMENSION(MAX_LAYERS) :: N_LAYER_MATL,N_LAYER_CELLS_MAX
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: MIN_DIFFUSIVITY
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: LAYER_THICKNESS,INTERNAL_HEAT_SOURCE
   REAL(EB), DIMENSION(MAX_LAYERS) :: LAYER_DENSITY,TMP_INNER,STRETCH_FACTOR,&
                                      MOISTURE_FRACTION,SURFACE_VOLUME_RATIO,PACKING_RATIO,KAPPA_S=-1._EB
   REAL(EB), DIMENSION(MAX_NUMBER_FSK_POINTS) :: FSK_K, FSK_W, FSK_A
   INTEGER :: NUMBER_FSK_POINTS
   CHARACTER(LABEL_LENGTH), ALLOCATABLE, DIMENSION(:) :: MATL_NAME
   CHARACTER(LABEL_LENGTH), DIMENSION(MAX_LAYERS,MAX_MATERIALS) :: LAYER_MATL_NAME
   REAL(EB), DIMENSION(MAX_LAYERS,MAX_MATERIALS) :: LAYER_MATL_FRAC
   LOGICAL :: BURN_AWAY,ADIABATIC,THERMALLY_THICK,INTERNAL_RADIATION,USER_DEFINED=.TRUE., &
              FREE_SLIP=.FALSE.,NO_SLIP=.FALSE.,SPECIFIED_NORMAL_VELOCITY=.FALSE.,SPECIFIED_TANGENTIAL_VELOCITY=.FALSE., &
              SPECIFIED_NORMAL_GRADIENT=.FALSE.,CONVERT_VOLUME_TO_MASS=.FALSE.,SPECIFIED_HEAT_SOURCE=.FALSE.,&
              THERMALLY_THICK_HT3D=.FALSE.,IMPERMEABLE=.FALSE.,BOUNDARY_FUEL_MODEL=.FALSE.
   INTEGER :: GEOMETRY,BACKING,PROFILE,HEAT_TRANSFER_MODEL=0
   CHARACTER(LABEL_LENGTH) :: PART_ID,RAMP_Q,RAMP_V,RAMP_T,RAMP_EF,RAMP_PART,RAMP_V_X,RAMP_V_Y,RAMP_V_Z,RAMP_T_B,RAMP_T_I
   CHARACTER(LABEL_LENGTH), ALLOCATABLE, DIMENSION(:) :: RAMP_MF
   CHARACTER(60) :: ID,TEXTURE_MAP
   CHARACTER(MESSAGE_LENGTH) :: FYI='null'

   ! Level Set Firespread
   LOGICAL :: VEG_LSET_SPREAD,VEG_LSET_ELLIPSE,VEG_LSET_TAN2
   REAL(EB) :: VEG_LSET_IGNITE_T,VEG_LSET_ROS_HEAD,VEG_LSET_ELLIPSE_HEAD,VEG_LSET_QCON,VEG_LSET_ROS_FLANK,VEG_LSET_ROS_BACK, &
               VEG_LSET_WIND_EXP

   !Veg SAV and height (fuel depth) for Farsite emulation
   REAL(EB) :: VEG_LSET_SIGMA,VEG_LSET_HT,VEG_LSET_BETA

   !HTC Custom
   REAL(EB) :: C_FORCED_CONSTANT=0._EB,C_FORCED_PR_EXP=0._EB,C_FORCED_RE=0._EB,C_FORCED_RE_EXP=0._EB,C_HORIZONTAL,C_VERTICAL

END TYPE SURFACE_TYPE

TYPE (SURFACE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: SURFACE

TYPE OMESH_TYPE
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:) :: MU,RHO,RHOS,TMP,U,V,W,US,VS,WS,H,HS,FVX,FVY,FVZ,D,DS,KRES,IL_S,IL_R,Q
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:,:) :: ZZ,ZZS
   INTEGER, ALLOCATABLE, DIMENSION(:) :: BOUNDARY_TYPE,IIO_R,JJO_R,KKO_R,IOR_R,IIO_S,JJO_S,KKO_S,IOR_S
   INTEGER, ALLOCATABLE, DIMENSION(:) :: N_PART_ORPHANS,N_PART_ADOPT,EXPOSED_WALL_CELL_BACK_INDICES,WALL_CELL_INDICES_SEND
   INTEGER :: I_MIN_R=-10,I_MAX_R=-10,J_MIN_R=-10,J_MAX_R=-10,K_MIN_R=-10,K_MAX_R=-10,NIC_R=0,N_WALL_CELLS_SEND=0, &
              I_MIN_S=-10,I_MAX_S=-10,J_MIN_S=-10,J_MAX_S=-10,K_MIN_S=-10,K_MAX_S=-10,NIC_S=0,N_EXPOSED_WALL_CELLS=0
   INTEGER, DIMENSION(7) :: INTEGER_SEND_BUFFER,INTEGER_RECV_BUFFER
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: &
         REAL_SEND_PKG1,REAL_SEND_PKG2,REAL_SEND_PKG3,REAL_SEND_PKG4,REAL_SEND_PKG5,REAL_SEND_PKG6,REAL_SEND_PKG7,&
         REAL_RECV_PKG1,REAL_RECV_PKG2,REAL_RECV_PKG3,REAL_RECV_PKG4,REAL_RECV_PKG5,REAL_RECV_PKG6,REAL_RECV_PKG7
   TYPE (STORAGE_TYPE), ALLOCATABLE, DIMENSION(:) :: ORPHAN_PARTICLE_STORAGE,ADOPT_PARTICLE_STORAGE
   TYPE (EXPOSED_WALL_TYPE), ALLOCATABLE, DIMENSION(:) :: EXPOSED_WALL

   ! CC_IBM data exchange arrays:
   INTEGER :: NICC_S(2)=0, NICC_R(2)=0, NFCC_S(2)=0, NFCC_R(2)=0, NCC_INT_R=0
   REAL(EB), ALLOCATABLE, DIMENSION(:) ::                &
         REAL_SEND_PKG11,REAL_SEND_PKG12,REAL_SEND_PKG13,&
         REAL_RECV_PKG11,REAL_RECV_PKG12,REAL_RECV_PKG13
   INTEGER, ALLOCATABLE, DIMENSION(:) :: ICC_UNKZ_CT_S, ICC_UNKZ_CC_S, ICC_UNKZ_CT_R, ICC_UNKZ_CC_R
   INTEGER, ALLOCATABLE, DIMENSION(:) :: UNKZ_CT_S, UNKZ_CC_S, UNKZ_CT_R, UNKZ_CC_R

   ! Face variables data (velocities):
   INTEGER, ALLOCATABLE, DIMENSION(:) :: IIO_FC_R,JJO_FC_R,KKO_FC_R,AXS_FC_R,IIO_FC_S,JJO_FC_S,KKO_FC_S,AXS_FC_S
   INTEGER, ALLOCATABLE, DIMENSION(:) :: IIO_CC_R,JJO_CC_R,KKO_CC_R,IIO_CC_S,JJO_CC_S,KKO_CC_S

   ! Level Set
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: PHI_LS,PHI1_LS,U_LS,V_LS,Z_LS
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: REAL_SEND_PKG14,REAL_RECV_PKG14

END TYPE OMESH_TYPE

TYPE OBSTRUCTION_TYPE
   CHARACTER(LABEL_LENGTH) :: DEVC_ID='null',CTRL_ID='null',PROP_ID='null',MATL_ID='null',ID='null'
   INTEGER, DIMENSION(-3:3) :: SURF_INDEX=0
   LOGICAL, DIMENSION(-3:3) :: SHOW_BNDF=.TRUE.
   INTEGER, DIMENSION(3) :: RGB=(/0,0,0/)
   INTEGER, DIMENSION(3) :: DIMENSIONS=0
   REAL(EB) :: TRANSPARENCY=1._EB,VOLUME_ADJUST=1._EB,BULK_DENSITY=-1._EB,INTERNAL_HEAT_SOURCE=0._EB
   REAL(EB), DIMENSION(3) :: TEXTURE=0._EB
   REAL(EB) :: X1=0._EB,X2=1._EB,Y1=0._EB,Y2=1._EB,Z1=0._EB,Z2=1._EB,MASS=1.E6_EB
   REAL(EB), DIMENSION(3) :: FDS_AREA=-1._EB,INPUT_AREA=-1._EB,UNDIVIDED_INPUT_AREA=-1._EB,SHAPE_AREA=0._EB
   INTEGER :: I1=-1,I2=-1,J1=-1,J2=-1,K1=-1,K2=-1,COLOR_INDICATOR=-1,TYPE_INDICATOR=-1,ORDINAL=0,SHAPE_TYPE=-1
   INTEGER :: DEVC_INDEX=-1,CTRL_INDEX=-1,PROP_INDEX=-1,DEVC_INDEX_O=-1,CTRL_INDEX_O=-1,MATL_INDEX=-1,MULT_INDEX=-1,&
              RAMP_Q_INDEX=0
   LOGICAL :: HIDDEN=.FALSE.,PERMIT_HOLE=.TRUE.,ALLOW_VENT=.TRUE.,CONSUMABLE=.FALSE.,REMOVABLE=.FALSE., &
              HOLE_FILLER=.FALSE.,NOTERRAIN=.FALSE.,OVERLAY=.TRUE.

   ! 3D pyrolysis:
   LOGICAL :: PYRO3D=.FALSE.,MT3D=.FALSE.,HT3D=.FALSE.,PYRO3D_LIQUID=.FALSE.
   INTEGER :: MATL_SURF_INDEX=-1,PYRO3D_IOR=0
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:,:) :: RHO
END TYPE OBSTRUCTION_TYPE

TYPE TRIBIN_TYPE
   REAL(EB):: X1_LOW, X1_HIGH
   INTEGER :: NTL
   INTEGER, ALLOCATABLE, DIMENSION(:) :: TRI_LIST
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:) :: XYZV
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:) :: FACECUBE
END TYPE TRIBIN_TYPE

TYPE TBAXIS_TYPE
   INTEGER :: N_BINS
   REAL(EB):: DELBIN
   TYPE(TRIBIN_TYPE), ALLOCATABLE, DIMENSION(:) :: TRIBIN
END TYPE TBAXIS_TYPE

TYPE TRANSFORM_TYPE
   CHARACTER(LABEL_LENGTH) :: GEOM_ID, ID
   CHARACTER(80) :: FILE
   REAL(EB) :: Z_OFFSET
   INTEGER :: GEOM_INDEX=-1
END TYPE TRANSFORM_TYPE

TYPE GEOMETRY_TYPE
   CHARACTER(LABEL_LENGTH) :: ID,MATL_ID,DEVC_ID,PROP_ID,MOVE_ID
   CHARACTER(LABEL_LENGTH), ALLOCATABLE, DIMENSION(:) :: SURF_ID
   CHARACTER(60) :: BNDC_FILENAME='null',GEOC_FILENAME='null',TEXTURE_MAPPING
   LOGICAL :: COMPONENT_ONLY,IS_DYNAMIC=.TRUE.,HAVE_SURF,HAVE_MATL,AUTO_TEXTURE,HIDDEN,REMOVEABLE,SHOW_BNDF=.TRUE., &
              READ_BINARY=.FALSE.,SNAP_TO_GRID=.FALSE.
   INTEGER :: N_VERTS_BASE,N_FACES_BASE,N_VOLUS_BASE,N_VERTS,N_EDGES,N_FACES,N_VOLUS,NSUB_GEOMS,GEOM_TYPE,IJK(3),N_LEVELS,&
              DEVC_INDEX=-1,CTRL_INDEX=-1,PROP_INDEX=-1,DEVC_INDEX_O=-1,CTRL_INDEX_O=-1,MATL_INDEX=-1,&
              CYLINDER_NSEG_THETA,CYLINDER_NSEG_AXIS
   REAL(EB) :: GEOM_VOLUME,GEOM_AREA,GEOM_XYZCEN(3),OMEGA=0._EB,XYZ0(3),AZIM_BASE,ELEV_BASE,SCALE_BASE(3),XYZ_BASE(3),&
               AZIM,ELEV,SCALE(3),XYZ(3),AZIM_DOT,ELEV_DOT,SCALE_DOT(3),XYZ_DOT(3),GAXIS(3),GROTATE,GROTATE_DOT,GROTATE_BASE,&
               XB(6),XB_ORIG(6),SPHERE_ORIGIN(3),SPHERE_RADIUS,TEXTURE_ORIGIN(3),TEXTURE_SCALE(2),MIN_LEDGE,MAX_LEDGE,MEAN_LEDGE,&
               GEOM_BOX(LOW_IND:HIGH_IND,IAXIS:KAXIS),TRANSPARENCY,CYLINDER_ORIGIN(3),CYLINDER_AXIS(3),&
               CYLINDER_RADIUS,CYLINDER_LENGTH
   INTEGER,ALLOCATABLE,DIMENSION(:) :: FACES,VOLUS,SUB_GEOMS,SURFS,MATLS
   INTEGER,ALLOCATABLE,DIMENSION(:,:) :: EDGES,FACE_EDGES,EDGE_FACES
   REAL(EB),ALLOCATABLE,DIMENSION(:) :: FACES_AREA,VERTS_BASE,VERTS,TFACES,DAZIM,DELEV,ZVALS
   REAL(EB),ALLOCATABLE,DIMENSION(:,:) :: FACES_NORMAL,DSCALE,DXYZ0,DXYZ
   REAL(EB),ALLOCATABLE, DIMENSION(:,:,:) :: FACECUBE
   TYPE(TBAXIS_TYPE) :: TBAXIS(IAXIS:KAXIS)
END TYPE GEOMETRY_TYPE

INTEGER :: N_GEOMETRY=0, GEOMETRY_CHANGE_STATE=0, N_TRNF=0
TYPE(GEOMETRY_TYPE),  ALLOCATABLE, TARGET, DIMENSION(:)   :: GEOMETRY
TYPE(GEOMETRY_TYPE),  ALLOCATABLE, TARGET, DIMENSION(:,:) :: GEOMETRY_TRANSFORM
TYPE(TRANSFORM_TYPE), ALLOCATABLE, TARGET, DIMENSION(:,:) :: TRANSFORM
LOGICAL :: IS_GEOMETRY_DYNAMIC

TYPE VERTEX_TYPE
   REAL(EB) :: X,Y,Z
   REAL(EB) :: XYZ(3) ! this form may be more efficient
END TYPE VERTEX_TYPE

TYPE(VERTEX_TYPE), TARGET, ALLOCATABLE, DIMENSION(:) :: VERTEX

! --- CC_IBM types:
! Edge crossings data structure:
INTEGER, PARAMETER :: IBM_MAXCROSS_EDGE = 10 ! Size definition parameter. Max number of crossings per Cartesian Edge.
TYPE IBM_EDGECROSS_TYPE
   INTEGER :: NCROSS   ! Number of BODINT_PLANE segments - Cartesian edge crossings.
   REAL(EB), DIMENSION(1:IBM_MAXCROSS_EDGE)   ::  SVAR ! Locations along x2 axis of NCROSS intersections.
   INTEGER,  DIMENSION(1:IBM_MAXCROSS_EDGE)   :: ISVAR ! Type of intersection (i.e. SG, GS or GG).
   INTEGER,  DIMENSION(MAX_DIM+1)             ::   IJK ! [ i j k X2AXIS]
END TYPE IBM_EDGECROSS_TYPE

! Cartesian Edge Cut-Edges data structure:
TYPE IBM_CUTEDGE_TYPE
   INTEGER :: NVERT, NEDGE, NEDGE1, STATUS         ! Local Vertices, cut-edges and status of this Cartesian edge.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           :: XYZVERT  ! Locations of vertices.
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)           ::  CEELEM  ! Cut-Edge connectivities.
   INTEGER,  DIMENSION(MAX_DIM+2)                  ::     IJK  ! [ i j k X2AXIS cetype]
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)           ::  INDSEG  ! [ntr tr1 tr2 ibod]
   INTEGER,  ALLOCATABLE, DIMENSION(:)             ::NOD_PERM  ! Permutation array for INSERT_FACE_VERT.
END TYPE IBM_CUTEDGE_TYPE

! REGC_EDGE type, used for computation of wall model turbulent viscosity, shear stress, vorticity.
TYPE IBM_RCEDGE_TYPE
   INTEGER,  DIMENSION(MAX_DIM+1)                  ::     IJK  ! [ i j k X1AXIS]
   INTEGER :: IE
   ! Fields related to IBM_PLANE_INTERPOLATION=.FALSE.:
   ! Here: VIND=IAXIS:KAXIS, EP=1:INT_N_EXT_PTS,
   ! INT_VEL_IND = 1; INT_VELS_IND = 2; INT_FV_IND = 3; INT_DHDX_IND = 4; N_INT_FVARS = 4;
   ! INT_NPE_LO = INT_NPE(LOW,VIND,EP,IFACE); INT_NPE_LO = INT_NPE(HIGH,VIND,EP,IEDGE).
   ! IEDGE = 0, Cartesian GASPHASE EDGE.
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_IJK        ! (IAXIS:KAXIS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:)        :: INT_COEF       ! (INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XYZBF      ! (IAXIS:KAXIS,IEDGE)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_INBFC      ! (1:3,IEDGE)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:,:,:)  :: INT_NPE        ! (LOW:HIGH,VIND,EP,IEDGE)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XN,INT_CN  ! (0:INT_N_EXT_PTS,IEDGE)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_FVARS      ! (1:N_INT_FVARS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_NOMIND     ! (LOW_IND:HIGH_IND,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
END TYPE IBM_RCEDGE_TYPE

! Cartesian Faces Cut-Faces data structure:
INTEGER, PARAMETER :: IBM_MAXVERTS_FACE  =1024 ! Size definition parameter. Max number of vertices per Cartesian Face.
INTEGER, PARAMETER :: IBM_MAXCEELEM_FACE =1024 ! Size definition parameter. Max segments per face.
INTEGER, PARAMETER :: IBM_MAXCFELEM_FACE =1024 ! Size definition parameter. Max number of cut faces per Cartesian Face.
INTEGER, PARAMETER :: IBM_MAXVERT_CUTFACE=  24 ! Size definition parameter.
INTEGER, PARAMETER :: MAX_INTERP_POINTS_PLANE = 4
TYPE IBM_CUTFACE_TYPE
   INTEGER :: IWC=0
   INTEGER :: NVERT=0, NSVERT=0, NFACE=0, NSFACE=0, STATUS !Local Vertices, cut-faces and status of this Cartesian face.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           :: XYZVERT  ! Locations of vertices.
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)           ::  CFELEM  ! Cut-faces connectivities.
   INTEGER,  DIMENSION(MAX_DIM+1)                  ::     IJK  ! [ i j k X1AXIS]
   REAL(EB), ALLOCATABLE, DIMENSION(:)             ::    AREA  ! Cut-faces areas.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           ::  XYZCEN  ! Cut-faces centroid locations.
   !Integrals to be used in cut-cell volume and centroid computations.
   REAL(EB), ALLOCATABLE, DIMENSION(:)             ::  INXAREA, INXSQAREA, JNYSQAREA, KNZSQAREA
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)           ::  BODTRI
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)           ::  UNKH, UNKZ
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           ::  XCENLOW, XCENHIGH
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           ::  RHO
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           ::  ZZ_FACE, DIFF_FACE, RHO_D, VELD
   REAL(EB), ALLOCATABLE, DIMENSION(:)             :: TMP_FACE
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:)         :: RHO_D_DZDN
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)           :: H_RHO_D_DZDN
   REAL(EB), ALLOCATABLE, DIMENSION(:)             ::  VEL, VELS, DHDX, FN, VELNP1, VELINT
   INTEGER,  ALLOCATABLE, DIMENSION(:,:,:)         ::  JDZ, JDH
   REAL(EB) :: VELN_CRF, VELD_CRF, DHDX_CRF, FN_CRF, VELNP1_CRF, VELINT_CRF
   INTEGER,  ALLOCATABLE, DIMENSION(:,:,:)                         ::      CELL_LIST ! [RC_TYPE I J K ]

   ! Here: VIND=IAXIS:KAXIS, EP=1:INT_N_EXT_PTS,
   ! INT_VEL_IND = 1; INT_VELS_IND = 2; INT_FV_IND = 3; INT_DHDX_IND = 4; N_INT_FVARS = 4;
   ! INT_NPE_LO = INT_NPE(LOW,VIND,EP,IFACE); INT_NPE_HI = INT_NPE(HIGH,VIND,EP,IFACE).
   ! IFACE = 0, undelying Cartesian face, IFACE=1:NFACE GASPHASE cut-faces.
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_IJK        ! (IAXIS:KAXIS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:)        :: INT_COEF       ! (INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_DCOEF      ! (IAXIS:KAXIS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XYZBF,INT_NOUT ! (IAXIS:KAXIS,0:NFACE)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_INBFC      ! (1:3,0:NFACE)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:,:,:)  :: INT_NPE        ! (LOW:HIGH,VIND,EP,0:NFACE)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XN,INT_CN  ! (0:INT_N_EXT_PTS,0:NFACE)  ! 0 is interpolation point.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_FVARS      ! (1:N_INT_FVARS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_NOMIND     ! (LOW_IND:HIGH_IND,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)

   ! Fields used in INBOUNDARY faces:
   ! Here: VIND=0, EP=1:INT_N_EXT_PTS,
   ! INT_H_IND=1, etc. N_INT_CVARS=INT_P_IND+N_TRACKED_SPECIES
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_CVARS      ! (1:N_INT_CVARS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)

   REAL(EB), ALLOCATABLE, DIMENSION(:,:)            :: RHOPVN
   INTEGER :: NOMICF(2)=0  ! [NOM icf]
   INTEGER,  ALLOCATABLE, DIMENSION(:)              :: CFACE_INDEX, SURF_INDEX
END TYPE IBM_CUTFACE_TYPE


TYPE RAD_CFACE_TYPE
   INTEGER :: N_ASSIGNED_CFACES_RADI=0
   INTEGER, ALLOCATABLE, DIMENSION(:) :: ASSIGNED_CFACES_RADI
END TYPE RAD_CFACE_TYPE

! Note: If you change the number of scalar variables in CFACE_TYPE, adjust the numbers below

INTEGER, PARAMETER :: N_CFACE_SCALAR_REALS=10,N_CFACE_SCALAR_INTEGERS=7,N_CFACE_SCALAR_LOGICALS=0

TYPE CFACE_TYPE
   TYPE (ONE_D_M_AND_E_XFER_TYPE) :: ONE_D
   REAL(EB), POINTER :: AREA,X,Y,Z,NVEC(:),VEL_ERR_NEW,V_DEP,Q_LEAK
   INTEGER, POINTER :: CFACE_INDEX,SURF_INDEX,NODE_INDEX,VENT_INDEX,BOUNDARY_TYPE,CUT_FACE_IND1,CUT_FACE_IND2
END TYPE CFACE_TYPE

! Cartesian Cells Cut-Cells data structure:
INTEGER, PARAMETER :: IBM_MAXVERTS_CELL   =1024
INTEGER, PARAMETER :: IBM_NPARAM_CCFACE   =  5 ! [face_type side iaxis cei icf]

TYPE IBM_CUTCELL_TYPE
   INTEGER :: NCELL, NFACE_CELL
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)                     ::    CCELEM ! Cut-cells faces connectivities in FACE_LIST.
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)                     :: FACE_LIST ! List of faces, cut-faces.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       ::    VOLUME ! Cut-cell volumes.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)                     ::    XYZCEN ! Cut-cell centroid locaitons.
   INTEGER,  DIMENSION(MAX_DIM)                              ::       IJK ! [ i j k ]
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       :: RHO, RHOS ! Cut cells densities.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       ::  RSUM,TMP ! Cut cells temperatures.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       ::    D,  DS ! Cut cell thermodynamic divg.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       ::Q,QR,D_SOURCE ! Q,Thermo divg reaction component.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       :: CHI_R,MIX_TIME ! Cut-cell combustion
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)                     ::    Q_REAC          ! variables.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)                     :: REAC_SOURCE_TERM   !
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)                     ::   ZZ, ZZS, M_DOT_PPP ! Cut cells species mass
                                                                                ! fractions and rho*D_z,reaction source.
   INTEGER,  ALLOCATABLE, DIMENSION(:)                       :: UNKH,UNKZ ! Unknown number for pressure H,
                                                                          ! and scalars.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       ::      H,HS ! Pressure H containers.
   REAL(EB), ALLOCATABLE, DIMENSION(:)                       ::  RTRM,R_H_G,RHO_0,WVEL

   ! Here: VIND=0, EP=1:INT_N_EXT_PTS
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_IJK        ! (IAXIS:KAXIS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:)        :: INT_COEF       ! (INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XYZBF,INT_NOUT ! (IAXIS:KAXIS,0:NCELL)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_INBFC      ! (1:3,0:NCELL)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:,:,:)  :: INT_NPE        ! (LOW:HIGH,0,EP,0:NCELL)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XN,INT_CN  ! (0:INT_N_EXT_PTS,0:NCELL)  ! 0 is interpolation point.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_CCVARS      ! (1:N_INT_CCVARS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_NOMIND     ! (LOW_IND:HIGH_IND,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)

   REAL(EB), ALLOCATABLE, DIMENSION(:,:)                     :: DEL_RHO_D_DEL_Z, U_DOT_DEL_RHO_Z
   LOGICAL,  ALLOCATABLE, DIMENSION(:)                       :: USE_CC_VOL
   INTEGER :: NOMICC(2)=0
   REAL(EB):: DIVVOL_BC=0._EB
END TYPE IBM_CUTCELL_TYPE


TYPE IBM_REGFACE_TYPE
   INTEGER,  DIMENSION(MAX_DIM)                                    ::       IJK
   INTEGER,  DIMENSION(1:2,1:2)                                    ::        JD
END TYPE IBM_REGFACE_TYPE

TYPE IBM_REGFACEZ_TYPE
   INTEGER :: IWC=0
   INTEGER,  DIMENSION(MAX_DIM)                                    ::       IJK
   INTEGER,  DIMENSION(1:2,1:2)                                    ::        JD
   REAL(EB), DIMENSION(MAX_SPECIES)                                ::   DIFF_FACE=0._EB, RHO_D=0._EB, VELD=0._EB
   REAL(EB), DIMENSION(MAX_SPECIES,LOW_IND:HIGH_IND)               ::   RHO_D_DZDN=0._EB
   REAL(EB), DIMENSION(MAX_SPECIES)                                :: H_RHO_D_DZDN=0._EB
   REAL(EB), DIMENSION(-1:0)                                       ::    RHOPVN=0._EB
END TYPE IBM_REGFACEZ_TYPE

TYPE IBM_RCFACE_TYPE
   INTEGER,  DIMENSION(MAX_DIM+1)                                  ::       IJK ! [ I J K x1axis]
   INTEGER,  DIMENSION(LOW_IND:HIGH_IND)                           ::       UNK
   REAL(EB), DIMENSION(MAX_DIM,LOW_IND:HIGH_IND)                   ::      XCEN
   INTEGER,  DIMENSION(1:2,1:2)                                    ::        JD
END TYPE IBM_RCFACE_TYPE

TYPE IBM_RCFACE_LST_TYPE
   INTEGER :: IWC=0
   REAL(EB):: TMP_FACE=0._EB
   INTEGER,  DIMENSION(MAX_DIM+1)                                  ::       IJK ! [ I J K x1axis]
   INTEGER,  DIMENSION(LOW_IND:HIGH_IND)                           ::       UNK
   REAL(EB), DIMENSION(MAX_DIM,LOW_IND:HIGH_IND)                   ::      XCEN
   INTEGER,  DIMENSION(1:2,1:2)                                    ::        JD
   INTEGER,  DIMENSION(MAX_DIM+1,LOW_IND:HIGH_IND)                 :: CELL_LIST ! [RC_TYPE I J K ]
   REAL(EB), DIMENSION(MAX_SPECIES)                              :: ZZ_FACE=0._EB,DIFF_FACE=0._EB,RHO_D=0._EB,VELD=0._EB
   REAL(EB), DIMENSION(MAX_SPECIES,LOW_IND:HIGH_IND)               :: RHO_D_DZDN=0._EB
   REAL(EB), DIMENSION(MAX_SPECIES)                                :: H_RHO_D_DZDN=0._EB
   REAL(EB), DIMENSION(-1:0)                                       ::    RHOPVN=0._EB
END TYPE IBM_RCFACE_LST_TYPE

TYPE IBM_EXIMFACE_TYPE
   INTEGER :: LHFACE, UNKZ, IWC=0
   INTEGER,  DIMENSION(MAX_DIM+1)                                  ::       IJK ! [ I J K x1axis]
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)                           ::       FLX
   REAL(EB) :: AREA,FN_H_S
   REAL(EB), DIMENSION(MAX_SPECIES)                                ::H_RHO_D_DZDN=0._EB,FN_ZZ=0._EB
END TYPE IBM_EXIMFACE_TYPE

! Velocity regular faces connected to cut-cell interpolation type:
INTEGER, PARAMETER :: MAX_RCVEL_NCFACE = 5
TYPE IBM_RCVEL_TYPE
   INTEGER :: NCFACE, IWC=0
   INTEGER,  DIMENSION(MAX_DIM+1)                                  ::            IJK ! [ I J K x1axis]
   INTEGER,  DIMENSION(MAX_DIM+1,LOW_IND:HIGH_IND,MAX_RCVEL_NCFACE)::      CELL_LIST ! [RC_TYPE I J K ]
   REAL(EB) :: VELINT

   ! Here: VIND=IAXIS:KAXIS, EP=1:INT_N_EXT_PTS,
   ! INT_VEL_IND = 1; INT_VELS_IND = 2; INT_FV_IND = 3; INT_DHDX_IND = 4; N_INT_FVARS = 4;
   ! INT_NPE_LO = INT_NPE(LOW,VIND,EP,IFACE); INT_NPE_LO = INT_NPE(HIGH,VIND,EP,IFACE).
   ! IFACE = 0 Cartesian GASPHASE face.
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_IJK        ! (IAXIS:KAXIS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:)        :: INT_COEF       ! (INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XYZBF, INT_NOUT ! (IAXIS:KAXIS,IFACE)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_INBFC      ! (1:3,IFACE)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:,:,:)  :: INT_NPE        ! (LOW:HIGH,VIND,EP,IFACE)
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_XN,INT_CN  ! (0:INT_N_EXT_PTS,IFACE) ! 0 is interpolation point.
   REAL(EB), ALLOCATABLE, DIMENSION(:,:)      :: INT_FVARS       ! (1:N_INT_FVARS,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)
   INTEGER,  ALLOCATABLE, DIMENSION(:,:)      :: INT_NOMIND     ! (LOW_IND:HIGH_IND,INT_NPE_LO+1:INT_NPE_LO+INT_NPE_HI)

END TYPE IBM_RCVEL_TYPE

TYPE CSVF_TYPE
    CHARACTER(255) :: CSVFILE,UVWFILE
END TYPE CSVF_TYPE

TYPE(CSVF_TYPE), ALLOCATABLE, DIMENSION(:) :: CSVFINFO

TYPE VENTS_TYPE
   INTEGER :: I1=-1,I2=-1,J1=-1,J2=-1,K1=-1,K2=-1,BOUNDARY_TYPE=0,IOR=0,SURF_INDEX=0,DEVC_INDEX=-1,CTRL_INDEX=-1, &
              COLOR_INDICATOR=99,TYPE_INDICATOR=0,ORDINAL=0,PRESSURE_RAMP_INDEX=0,TMP_EXTERIOR_RAMP_INDEX=0,NODE_INDEX=-1, &
              OBST_INDEX=0
   INTEGER, DIMENSION(3) :: RGB=-1
   REAL(EB) :: TRANSPARENCY = 1._EB
   REAL(EB), DIMENSION(3) :: TEXTURE=0._EB
   REAL(EB) :: X1=0._EB,X2=0._EB,Y1=0._EB,Y2=0._EB,Z1=0._EB,Z2=0._EB,FDS_AREA=0._EB, &
               X1_ORIG=0._EB,X2_ORIG=0._EB,Y1_ORIG=0._EB,Y2_ORIG=0._EB,Z1_ORIG=0._EB,Z2_ORIG=0._EB, &
               X0=-9.E6_EB,Y0=-9.E6_EB,Z0=-9.E6_EB,FIRE_SPREAD_RATE,UNDIVIDED_INPUT_AREA=0._EB,INPUT_AREA=0._EB,&
               TMP_EXTERIOR=-1000._EB,DYNAMIC_PRESSURE=0._EB,UVW(3)=-1.E12_EB,RADIUS=-1._EB
   LOGICAL :: ACTIVATED=.TRUE.,GHOST_CELLS_ONLY=.FALSE.,GEOM=.FALSE.
   CHARACTER(LABEL_LENGTH) :: DEVC_ID='null',CTRL_ID='null',ID='null'
   ! turbulent inflow (experimental)
   INTEGER :: N_EDDY=0
   REAL(EB) :: R_IJ(3,3)=0._EB,A_IJ(3,3)=0._EB,SIGMA_IJ(3,3)=0._EB,EDDY_BOX_VOLUME=0._EB, &
               X_EDDY_MIN=0._EB,X_EDDY_MAX=0._EB, &
               Y_EDDY_MIN=0._EB,Y_EDDY_MAX=0._EB, &
               Z_EDDY_MIN=0._EB,Z_EDDY_MAX=0._EB
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: U_EDDY,V_EDDY,W_EDDY
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: X_EDDY,Y_EDDY,Z_EDDY,CU_EDDY,CV_EDDY,CW_EDDY
END TYPE VENTS_TYPE

TYPE TABLES_TYPE
   INTEGER :: NUMBER_ROWS,NUMBER_COLUMNS
   REAL(EB) :: LX,LY,UX,UY
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: X,Y
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: TABLE_DATA,Z
END TYPE TABLES_TYPE

TYPE (TABLES_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: TABLES

TYPE RAMPS_TYPE
   REAL(EB) :: SPAN,RDT,T_MIN,T_MAX
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: INDEPENDENT_DATA,DEPENDENT_DATA,INTERPOLATED_DATA
   INTEGER :: NUMBER_DATA_POINTS,NUMBER_INTERPOLATION_POINTS,DEVC_INDEX=-1,CTRL_INDEX=-1
   CHARACTER(LABEL_LENGTH) :: DEVC_ID='null',CTRL_ID='null'
   LOGICAL :: DEP_VAR_UNITS_CONVERTED=.FALSE.,RESERVED=.FALSE.
END TYPE RAMPS_TYPE

TYPE (RAMPS_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: RAMPS

TYPE RESERVED_RAMPS_TYPE
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: INDEPENDENT_DATA,DEPENDENT_DATA
   INTEGER :: NUMBER_DATA_POINTS
END TYPE RESERVED_RAMPS_TYPE

INTEGER :: N_RESERVED_RAMPS=0
TYPE (RESERVED_RAMPS_TYPE), DIMENSION(10), TARGET :: RESERVED_RAMPS


TYPE HUMAN_TYPE
   CHARACTER(60) :: NODE_NAME='null'
   CHARACTER(LABEL_LENGTH) :: FFIELD_NAME='null'
   REAL(EB) :: X=0._EB,Y=0._EB,Z=0._EB,U=0._EB,V=0._EB,W=0._EB,F_X=0._EB,F_Y=0._EB,&
               X_old=0._EB,Y_old=0._EB,X_group=0._EB,Y_group=0._EB, U_CB=0.0_EB, V_CB=0.0_EB
   REAL(EB) :: UBAR=0._EB, VBAR=0._EB, UBAR_Center=0._EB, VBAR_Center=0._EB, T_LastRead_CB=0.0_EB
   REAL(EB) :: Speed=1.25_EB, Radius=0.255_EB, Mass=80.0_EB, Tpre=1._EB, Tau=1._EB, &
               Eta=0._EB, Ksi=0._EB, Tdet=0._EB, Speed_ave=0._EB, T_Speed_ave=0._EB, &
               X_CB=0.0_EB, Y_CB=0.0_EB, X2_CB=0.0_EB, Y2_CB=0.0_EB, UAVE_CB=0.0_EB, VAVE_CB=0.0_EB
   REAL(EB) :: r_torso=0.15_EB, r_shoulder=0.095_EB, d_shoulder=0.055_EB, angle=0._EB, &
               torque=0._EB, m_iner=4._EB
   REAL(EB) :: tau_iner=0.2_EB, angle_old=0._EB, omega=0._EB
   REAL(EB) :: A=2000._EB, B=0.08_EB, C_Young=120000._EB, Gamma=16000._EB, Kappa=40000._EB, &
               Lambda=0.5_EB, Commitment=0._EB
   REAL(EB) :: SumForces=0._EB, IntDose=0._EB, DoseCrit1=0._EB, DoseCrit2=0._EB, SumForces2=0._EB
   REAL(EB) :: TempMax1=0._EB, FluxMax1=0._EB, TempMax2=0._EB, FluxMax2=0._EB, Density=0._EB, DensityR=0._EB, DensityL=0._EB
   REAL(EB) :: P_detect_tot=0._EB, v0_fac=1._EB, D_Walls=0._EB
   REAL(EB) :: T_FallenDown=0._EB, F_FallDown=0._EB, Angle_FallenDown=0._EB, SizeFac_FallenDown=0._EB, T_CheckFallDown=0._EB
   INTEGER  :: IOR=-1, ILABEL=0, COLOR_INDEX=0, INODE=0, IMESH=-1, IPC=0, IEL=0, I_FFIELD=0, I_Target2=0, ID_CB=-1
   INTEGER  :: GROUP_ID=0, DETECT1=0, GROUP_SIZE=0, I_Target=0, I_DoorAlgo=0, I_Door_Mode=0, STRS_Direction = 1
   INTEGER  :: STR_SUB_INDX, SKIP_WALL_FORCE_IOR
   LOGICAL  :: SHOW=.TRUE., NewRnd=.TRUE., CROWBAR_READ_IN=.FALSE., CROWBAR_UPDATE_V0=.FALSE.
   LOGICAL  :: SeeDoorXB1=.FALSE., SeeDoorXB2=.FALSE., SeeDoorXYZ1=.FALSE., SeeDoorXYZ2=.FALSE.
END TYPE HUMAN_TYPE

TYPE HUMAN_GRID_TYPE
! (x,y,z) Centers of the grid cells in the main evacuation meshes
! SOOT_DENS: Smoke density at the center of the cell (mg/m3)
! FED_CO_CO2_O2: Purser's FED for co, co2, and o2
   REAL(EB) :: X,Y,Z,SOOT_DENS,FED_CO_CO2_O2,TMP_G,RADFLUX
   INTEGER :: N, N_old, IGRID, IHUMAN, ILABEL
! IMESH: (x,y,z) which fire mesh, if any
! II,JJ,KK: Fire mesh cell reference
   INTEGER  :: IMESH,II,JJ,KK
END TYPE HUMAN_GRID_TYPE

TYPE SLICE_TYPE
   INTEGER :: I1,I2,J1,J2,K1,K2,IOR=0,GEOM_INDEX=-1,TRNF_INDEX=-1,INDEX,INDEX2=0,Z_INDEX=-999,Y_INDEX=-999,MATL_INDEX=-999,&
              PART_INDEX=0,VELO_INDEX=0,PROP_INDEX=0,REAC_INDEX=0,SLCF_INDEX
   REAL(FB), DIMENSION(2) :: MINMAX
   REAL(FB) :: RLE_MIN, RLE_MAX
   REAL(EB):: AGL_SLICE
   LOGICAL :: TERRAIN_SLICE=.FALSE.,CELL_CENTERED=.FALSE.,FACE_CENTERED=.FALSE.,RLE=.FALSE.
   CHARACTER(60) :: SLICETYPE='STRUCTURED',SMOKEVIEW_LABEL
   CHARACTER(LABEL_LENGTH) :: SMOKEVIEW_BAR_LABEL,ID='null',MATL_ID='null',TRNF_ID='null'
END TYPE SLICE_TYPE

TYPE RAD_FILE_TYPE
   INTEGER :: I1,I2,J1,J2,K1,K2,I_STEP=1,J_STEP=1,K_STEP=1,N_POINTS
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:,:) :: IL_SAVE
END TYPE RAD_FILE_TYPE

TYPE PATCH_TYPE
   INTEGER :: I1,I2,J1,J2,K1,K2,IG1,IG2,JG1,JG2,KG1,KG2,IOR,OBST_INDEX
END TYPE PATCH_TYPE

TYPE BOUNDARY_FILE_TYPE
   INTEGER :: DEBUG=0,INDEX,PROP_INDEX,Z_INDEX=-999,Y_INDEX=-999,PART_INDEX=0,TIME_INTEGRAL_INDEX=0
   CHARACTER(60) :: SMOKEVIEW_LABEL
   CHARACTER(LABEL_LENGTH) :: SMOKEVIEW_BAR_LABEL,UNITS,MATL_ID='null'
   LOGICAL :: CELL_CENTERED=.FALSE.
END TYPE BOUNDARY_FILE_TYPE

TYPE (BOUNDARY_FILE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: BOUNDARY_FILE

TYPE BOUNDARY_ELEMENT_FILE_TYPE
   INTEGER :: INDEX,PROP_INDEX,Z_INDEX=-999,Y_INDEX=-999,PART_INDEX=0
   CHARACTER(60) :: SMOKEVIEW_LABEL
   CHARACTER(LABEL_LENGTH) :: SMOKEVIEW_BAR_LABEL
   LOGICAL :: CELL_CENTERED=.TRUE.
END TYPE BOUNDARY_ELEMENT_FILE_TYPE

TYPE (BOUNDARY_ELEMENT_FILE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: BOUNDARY_ELEMENT_FILE

TYPE ISOSURFACE_FILE_TYPE
   INTEGER :: INDEX=1,N_VALUES=1,Y_INDEX=-999,Z_INDEX=-999,VELO_INDEX=0,SKIP=1,&
              INDEX2=-1,Y_INDEX2=-999,Z_INDEX2=-999,VELO_INDEX2=0
   REAL(FB) :: VALUE(10), DELTA
   CHARACTER(60) :: SMOKEVIEW_LABEL, SMOKEVIEW_LABEL2
   CHARACTER(LABEL_LENGTH) :: SMOKEVIEW_BAR_LABEL, SMOKEVIEW_BAR_LABEL2
END TYPE ISOSURFACE_FILE_TYPE

TYPE (ISOSURFACE_FILE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: ISOSURFACE_FILE

TYPE PROFILE_TYPE
   REAL(EB) :: X,Y,Z
   INTEGER  :: IOR=0,WALL_INDEX=0,LP_TAG=0,ORDINAL,MESH,FORMAT_INDEX=1,PART_CLASS_INDEX=-1
   CHARACTER(LABEL_LENGTH) :: ID='null',QUANTITY='TEMPERATURE',INIT_ID='null'
END TYPE PROFILE_TYPE

TYPE (PROFILE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: PROFILE

TYPE INITIALIZATION_TYPE
   REAL(EB) :: TEMPERATURE,DENSITY,X1,X2,Y1,Y2,Z1,Z2,MASS_PER_VOLUME,MASS_PER_TIME,DT_INSERT,T_INSERT, &
               X0,Y0,Z0,U0,V0,W0,VOLUME,HRRPUV=0._EB,DX=0._EB,DY=0._EB,DZ=0._EB,HEIGHT,RADIUS,DIAMETER=-1._EB, &
               PARTICLE_WEIGHT_FACTOR, PACKING_RATIO
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: PARTICLE_INSERT_CLOCK,MASS_FRACTION
   INTEGER  :: PART_INDEX=0,N_PARTICLES,LU_PARTICLE,PROF_INDEX=0,DEVC_INDEX=0,CTRL_INDEX=0,TABL_INDEX=0, &
               N_PARTICLES_PER_CELL=0,PATH_RAMP_INDEX(3)=0,RAMP_Q_INDEX=0
   LOGICAL :: ADJUST_DENSITY=.FALSE.,ADJUST_TEMPERATURE=.FALSE.,SINGLE_INSERTION=.TRUE., &
              CELL_CENTERED=.FALSE.,UNIFORM=.FALSE.
   LOGICAL, ALLOCATABLE, DIMENSION(:) :: ALREADY_INSERTED
   CHARACTER(LABEL_LENGTH) :: SHAPE,DEVC_ID,CTRL_ID,ID
END TYPE INITIALIZATION_TYPE

TYPE (INITIALIZATION_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: INITIALIZATION

TYPE INIT_RESERVED_TYPE
   REAL(EB) :: XYZ(3),DX,DY,DZ
   INTEGER :: N_PARTICLES,DEVC_INDEX
   CHARACTER(LABEL_LENGTH) :: DEVC_ID
END TYPE INIT_RESERVED_TYPE

TYPE (INIT_RESERVED_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: INIT_RESERVED

TYPE P_ZONE_TYPE
   REAL(EB) :: X,Y,Z,DPSTAR=0._EB
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: LEAK_AREA,LEAK_REFERENCE_PRESSURE,LEAK_PRESSURE_EXPONENT
   INTEGER :: N_DUCTNODES,MESH_INDEX=0
   LOGICAL :: EVACUATION=.FALSE.,PERIODIC=.FALSE.
   INTEGER, ALLOCATABLE, DIMENSION(:) :: NODE_INDEX
   CHARACTER(LABEL_LENGTH) :: ID
END TYPE P_ZONE_TYPE

TYPE (P_ZONE_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: P_ZONE

TYPE MOVEMENT_TYPE
   REAL(EB) :: DX,DY,DZ,X0,Y0,Z0,ROTATION_ANGLE,AXIS(3),T34(3,4)
   INTEGER :: INDEX
   CHARACTER(LABEL_LENGTH) :: ID
END TYPE MOVEMENT_TYPE

TYPE (MOVEMENT_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: MOVEMENT

TYPE MULTIPLIER_TYPE
   REAL(EB) :: DXB(6),DX0,DY0,DZ0,FDS_AREA(6)=0._EB
   INTEGER  :: I_LOWER,I_UPPER,J_LOWER,J_UPPER,K_LOWER,K_UPPER,N_LOWER,N_UPPER,N_COPIES
   LOGICAL, ALLOCATABLE, DIMENSION(:,:,:) :: SKIP
   CHARACTER(LABEL_LENGTH) :: ID
   LOGICAL :: SEQUENTIAL=.FALSE.
END TYPE MULTIPLIER_TYPE

TYPE (MULTIPLIER_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: MULTIPLIER

TYPE FILTER_TYPE
   INTEGER :: RAMP_INDEX = -1
   CHARACTER(LABEL_LENGTH) :: ID,TABLE_ID
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: EFFICIENCY,MULTIPLIER
   REAL(EB) :: CLEAN_LOSS,LOADING_LOSS
END TYPE FILTER_TYPE

TYPE (FILTER_TYPE), DIMENSION(:), ALLOCATABLE,  TARGET :: FILTER

TYPE DUCTNODE_TYPE
   INTEGER :: FILTER_INDEX=-1, N_DUCTS,VENT_INDEX = -1, MESH_INDEX = -1,ZONE_INDEX=-1
   INTEGER, ALLOCATABLE, DIMENSION(:) :: DUCT_INDEX
   CHARACTER(LABEL_LENGTH) :: ID,TABLE_ID,VENT_ID='null'
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: LOSS_ARRAY,FILTER_LOADING
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: ZZ,DIR,ZZ_V
   REAL(EB) :: LOSS, P,P_OLD,TMP=273.15_EB,RHO,RSUM,CP,XYZ(3),FILTER_LOSS,TMP_V,RHO_V,RSUM_V,CP_V
   LOGICAL :: UPDATED, READ_IN, FIXED, AMBIENT = .FALSE.,LEAKAGE=.FALSE.,VENT=.FALSE.
   LOGICAL, ALLOCATABLE, DIMENSION(:) :: IN_MESH
END TYPE DUCTNODE_TYPE

TYPE (DUCTNODE_TYPE), DIMENSION(:), ALLOCATABLE,  TARGET :: DUCTNODE

TYPE NODE_BC_TYPE
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: ZZ_V
   REAL(EB) :: TMP_V,RHO_V,RSUM_V,CP_V, P
END TYPE NODE_BC_TYPE

TYPE DUCT_TYPE
   INTEGER :: NODE_INDEX(2)=-1,DEVC_INDEX=-1,CTRL_INDEX=-1,FAN_INDEX=-1,AIRCOIL_INDEX=-1,RAMP_INDEX=0,RAMP_LOSS_INDEX=0,&
              N_CELLS=1,DUCT_INTERP_TYPE_INDEX,SURF_INDEX=-1,LPC_INDEX=-1,QFAN_N=-1
   REAL(EB) :: AREA,AREA_INITIAL,COIL_Q=0._EB,CP_D,DIAMETER,DP_FAN(2)=0._EB,DX=-1._EB,LENGTH,MASS_FLOW_INITIAL=1.E6_EB,&
               RHO_D,ROUGHNESS,RSUM_D=0._EB,TAU=-1._EB,TMP_D=273.15_EB,TOTAL_LOSS=0._EB,VEL(4)=0._EB,VOLUME_FLOW=1.E6_EB,&
               VOLUME_FLOW_INITIAL=1.E6_EB,LOSS(2)=0._EB
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: CP_C,RHO_C,TMP_C,ZZ,PART_INDEX,PART_CELL_INDEX
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: ZZ_C
   REAL(EB), ALLOCATABLE, DIMENSION(:,:,:) :: VEL_SYSTEM
   LOGICAL :: ROUND = .TRUE.,SQUARE = .FALSE.,DAMPER = .FALSE.,DAMPER_OPEN = .TRUE.,FAN_OPERATING=.TRUE.,COIL_OPERATING=.TRUE.,&
              FIXED=.FALSE.,REVERSE=.FALSE.,UPDATED,LEAKAGE=.FALSE.,LEAK_ENTHALPY=.FALSE.,LOCALIZED_LEAKAGE=.FALSE.
   CHARACTER(LABEL_LENGTH) :: ID
   REAL(EB) :: FAN_ON_TIME = 1.E10_EB,COIL_ON_TIME = 1.E10_EB
END TYPE DUCT_TYPE

TYPE (DUCT_TYPE), DIMENSION(:), ALLOCATABLE,  TARGET :: DUCT

TYPE FAN_TYPE
   INTEGER :: FAN_TYPE,RAMP_INDEX,SPIN_INDEX=0
   REAL(EB) :: VOL_FLOW,MAX_FLOW,MAX_PRES,OFF_LOSS=0._EB,TAU=0._EB
   CHARACTER(LABEL_LENGTH) :: ID,FAN_RAMP
END TYPE FAN_TYPE

TYPE(FAN_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: FAN

TYPE AIRCOIL_TYPE
   REAL(EB) :: COOLANT_TEMPERATURE=293.15_EB,COOLANT_SPECIFIC_HEAT=4186._EB, EFFICIENCY, COOLANT_MASS_FLOW=-9999._EB, &
               FIXED_Q=-1.E10_EB,TAU=0._EB
   INTEGER :: RAMP_INDEX=0
   CHARACTER(LABEL_LENGTH) :: ID
END TYPE AIRCOIL_TYPE

TYPE(AIRCOIL_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: AIRCOIL

TYPE NETWORK_TYPE
   INTEGER :: N_DUCTS,N_DUCTNODES,N_MATRIX
   INTEGER, ALLOCATABLE, DIMENSION(:) :: DUCT_INDEX,NODE_INDEX,MATRIX_INDEX
END TYPE NETWORK_TYPE

TYPE(NETWORK_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: NETWORK

TYPE DUCTRUN_TYPE
   INTEGER :: N_DUCTS,N_DUCTNODES,DUCT_INTERP_TYPE_INDEX,N_MATRIX_SYSTEM=-1,N_QFANS=-1,N_FANS
   INTEGER, ALLOCATABLE, DIMENSION(:) :: DUCT_INDEX,NODE_INDEX,MATRIX_SYSTEM_INDEX,QFAN_INDEX
   REAL(EB), ALLOCATABLE, DIMENSION(:,:) :: RHS_SYSTEM_2,LHS_SYSTEM_1,LHS_SYSTEM_2
   REAL(EB), ALLOCATABLE, DIMENSION(:) :: RHS_SYSTEM_1
END TYPE DUCTRUN_TYPE

TYPE(DUCTRUN_TYPE), DIMENSION(:), ALLOCATABLE, TARGET :: DUCTRUN


END MODULE TYPES
