% McDermott
% 2-24-10
% sandia_methane_fire.m
%
% Plot Sandia 1m methane pool fire data and FDS results

close all
clear all

disp('sandia_methane_fire ...')

expdir = '../../../exp/Sandia_Plumes/';
outdir = '../../../out/Sandia_Plumes/';
pltdir = '../../Manuals/FDS_Validation_Guide/SCRIPT_FIGURES/Sandia_Plumes/';

% plot velocity signal and power spectrum

velocity_signal([outdir,'Sandia_CH4_1m_Test17_dx1p5cm_devc.csv'],2,'k-',10,20,0,7,'Time (s)','Vertical Velocity (m/s)', ...
                'Sandia Methane Pool Fire, Test 17','{\itx} = 0.0 m, {\itz} = 0.5 m', ...
                [pltdir,'Sandia_CH4_1m_Test17_dx1p5cm_velsignal'], ...
                [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_git.txt']);

power_spectrum([pltdir,'Sandia_CH4_1m_Test17_dx1p5cm_powerspectrum'],0.61,1,'Frequency (Hz)','Autospectral Density (m^2/s)',1e-1,1e3,1e-6,1e1, ...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_devc.csv'],[2:5],10,20,.00223,'k-','k--',...
               'Sandia 1 m Methane Pool Fire, Test 17','FDS W-Velocity, 1.5 cm Resolution', ...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_git.txt']);

exp_stride = 4;

% Test 14, Low Flow Rate

% vertical velocity

radial_profile([pltdir,'Sandia_CH4_1m_Test14_W_zp3'],'col',2,0.07,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 14, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test14_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test14_W_zp3.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test14_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test14_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test14_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test14_W_zp5'],'col',3,0.07,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 14, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test14_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test14_W_zp5.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test14_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test14_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test14_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test14_W_zp9'],'col',4,0.07,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 14, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test14_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test14_W_zp9.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test14_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test14_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test14_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

% radial velocity

radial_profile([pltdir,'Sandia_CH4_1m_Test14_U_zp3'],'col',5,0.13,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 14, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test14_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test14_U_zp3.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test14_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test14_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test14_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test14_U_zp5'],'col',6,0.13,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 14, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test14_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test14_U_zp5.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test14_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test14_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test14_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test14_U_zp9'],'col',7,0.13,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 14, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test14_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test14_U_zp9.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test14_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test14_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test14_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

% Test 17, High Flow Rate

% vertical velocity

radial_profile([pltdir,'Sandia_CH4_1m_Test17_W_zp3'],'col',2,0.21,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 17, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test17_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test17_W_zp3.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test17_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test17_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test17_W_zp5'],'col',3,0.21,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 17, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test17_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test17_W_zp5.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test17_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test17_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test17_W_zp9'],'col',4,0.21,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 17, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test17_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test17_W_zp9.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test17_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test17_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

% radial velocity

radial_profile([pltdir,'Sandia_CH4_1m_Test17_U_zp3'],'col',5,0.23,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 17, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test17_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test17_U_zp3.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test17_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test17_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test17_U_zp5'],'col',6,0.23,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 17, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test17_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test17_U_zp5.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test17_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test17_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test17_U_zp9'],'col',7,0.23,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 17, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test17_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test17_U_zp9.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test17_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test17_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test17_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')


% Test 24, Medium Flow Rate

% vertical velocity

radial_profile([pltdir,'Sandia_CH4_1m_Test24_W_zp3'],'col',2,0.17,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_W_zp3.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test24_W_zp5'],'col',3,0.17,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_W_zp5.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test24_W_zp9'],'col',4,0.17,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_W_zp9.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

% radial velocity

radial_profile([pltdir,'Sandia_CH4_1m_Test24_U_zp3'],'col',5,0.17,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_U_zp3.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test24_U_zp5'],'col',6,0.17,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_U_zp5.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test24_U_zp9'],'col',7,0.17,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_U_zp9.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')


% turbulent kinetic energy

radial_profile([pltdir,'Sandia_CH4_1m_Test24_TKE_p3'],'col_tke',[8,11],0.16,'relative',10, ...
               'Radial Position (m)','TKE (m^2/s^2)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,5,1,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_TKE_p3.csv'],'mo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'m-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'m--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'m-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test24_TKE_p5'],'col_tke',[9,12],0.16,'relative',10, ...
               'Radial Position (m)','TKE (m^2/s^2)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,5,1,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_TKE_p5.csv'],'mo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'m-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'m--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'m-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_CH4_1m_Test24_TKE_p9'],'col_tke',[10,13],0.16,'relative',10, ...
               'Radial Position (m)','TKE (m^2/s^2)','Sandia Methane Pool Fire','Test 24, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,5,1,[outdir,'Sandia_CH4_1m_Test24_dx6cm_git.txt'], ...
               [expdir,'Sandia_CH4_1m_Test24_TKE_p9.csv'],'mo','Exp',exp_stride, ...
               [outdir,'Sandia_CH4_1m_Test24_dx6cm_line.csv'],'m-.','FDS 6 cm', ...
               [outdir,'Sandia_CH4_1m_Test24_dx3cm_line.csv'],'m--','FDS 3 cm',...
               [outdir,'Sandia_CH4_1m_Test24_dx1p5cm_line.csv'],'m-','FDS 1.5 cm')




% Test 35, Hydrogen

% vertical velocity

radial_profile([pltdir,'Sandia_H2_1m_Test35_W_zp3'],'col',2,0.08,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_W_zp3.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_H2_1m_Test35_W_zp5'],'col',3,0.08,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Hyrdogen Pool Fire','Test 35, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_W_zp5.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_H2_1m_Test35_W_zp9'],'col',4,0.08,'relative',10, ...
               'Radial Position (m)','Vertical Velocity (m/s)','Sandia Hyrdogen Pool Fire','Test 35, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_W_zp9.csv'],'bo','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'b-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'b--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'b-','FDS 1.5 cm')

% radial velocity

radial_profile([pltdir,'Sandia_H2_1m_Test35_U_zp3'],'col',5,0.16,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_U_zp3.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_H2_1m_Test35_U_zp5'],'col',6,0.16,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_U_zp5.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_H2_1m_Test35_U_zp9'],'col',7,0.16,'relative',10, ...
               'Radial Position (m)','Radial Velocity (m/s)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,-.8,.8,.2,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_U_zp9.csv'],'ro','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'r-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'r--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'r-','FDS 1.5 cm')

% turbulent kinetic energy

radial_profile([pltdir,'Sandia_H2_1m_Test35_TKE_p3'],'col_tke',[8,11],0.16,'relative',10, ...
               'Radial Position (m)','TKE (m^2/s^2)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.3 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_TKE_p3.csv'],'mo','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'m-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'m--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'m-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_H2_1m_Test35_TKE_p5'],'col_tke',[9,12],0.16,'relative',10, ...
               'Radial Position (m)','TKE (m^2/s^2)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.5 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_TKE_p5.csv'],'mo','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'m-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'m--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'m-','FDS 1.5 cm')

radial_profile([pltdir,'Sandia_H2_1m_Test35_TKE_p9'],'col_tke',[10,13],0.16,'relative',10, ...
               'Radial Position (m)','TKE (m^2/s^2)','Sandia Hydrogen Pool Fire','Test 35, {\itz} = 0.9 m','Northeast', ...
               -.5,.5,.05,-.5,.5,.1,0,8,1,[outdir,'Sandia_H2_1m_Test35_dx6cm_git.txt'], ...
               [expdir,'Sandia_H2_1m_Test35_TKE_p9.csv'],'mo','Exp',exp_stride, ...
               [outdir,'Sandia_H2_1m_Test35_dx6cm_line.csv'],'m-.','FDS 6 cm', ...
               [outdir,'Sandia_H2_1m_Test35_dx3cm_line.csv'],'m--','FDS 3 cm',...
               [outdir,'Sandia_H2_1m_Test35_dx1p5cm_line.csv'],'m-','FDS 1.5 cm')








